import pandas as pd
import numpy as np

class Player:
    professional_yardages = {
    "Driver": 280,
    "3 Wood": 240,
    "5 Wood": 225,
    "4 Iron": 210,
    "5 Iron": 195,
    "6 Iron": 180,
    "7 Iron": 165,
    "8 Iron": 150,
    "9 Iron": 135,
    "Pitching Wedge": 120,
    "Sand Wedge": 21,
    "Putter": 20
    }

    def __init__(self, name, skill, handicap=None,score=None):
        self.name = name
        self.handicap = handicap
        self.skill = skill
        self.score= score
        self.yardages = self.calculate_yardages(skill)

    #Skill level 1 = pro
    def calculate_yardages (self,skill):
        beginner_multiplier = 0.6  # 60% of professional yardage
        intermediate_multiplier = 0.8  # 80% of professional yardage
        if skill == 1:
            return Player.professional_yardages
        elif skill == 2:
            return {club: int(yardage * intermediate_multiplier) for club, yardage in Player.professional_yardages.items()}
        elif skill == 3: 
            return {club: int(yardage * beginner_multiplier) for club, yardage in Player.professional_yardages.items()}

    def __str__(self):
        skill_level = "Professional" if self.skill == 1 else "Intermediate" if self.skill == 2 else "Amateur"
        output = f"\nPlayer: {self.name}\nSkill Level: {skill_level}\nHandicap: {self.handicap if self.handicap is not None else 'N/A'}\n"
        if self.score is not None:
            output += "Score:\n" + str(pd.DataFrame([self.score], index=[self.name]))
        return output




from golf_companion_533_app.__course_class import Course 
import unittest
import os
import pandas as pd


class TestCourseOkanagan(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        current_directory = os.path.dirname(os.path.abspath(__file__))
        cls.csv_file_path = os.path.join(current_directory, "course_files", "okanagan_golf_club_bear.csv")

    def setUp(self):
        self.course = Course("Okanagan Golf Club Bear", TestCourseOkanagan.csv_file_path, 68)

    def test_course_name(self):
        self.assertEqual(self.course.course_name, "Okanagan Golf Club Bear")

    def test_course_record(self):
        self.assertEqual(self.course.course_record, 68)

    def test_course_par(self):
        expected_par = self.course.score_card['par'].sum()
        self.assertEqual(self.course.par, expected_par)

    def test_course_score_card_type(self):
        self.assertIsInstance(self.course.score_card, pd.DataFrame)


class TestCourseShadowRidge(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        current_directory = os.path.dirname(os.path.abspath(__file__))
        cls.csv_file_path = os.path.join(current_directory, "course_files", "okanagan_golf_club_bear.csv")

    def setUp(self):
        self.course = Course("Shadow Ridge", TestCourseShadowRidge.csv_file_path, 64)

    def test_course_name(self):
        self.assertEqual(self.course.course_name, "Shadow Ridge")

    def test_course_record(self):
        self.assertEqual(self.course.course_record, 64)

    def test_course_par(self):
        expected_par = self.course.score_card['par'].sum()
        self.assertEqual(self.course.par, expected_par)

    def test_course_score_card_type(self):
        self.assertIsInstance(self.course.score_card, pd.DataFrame)

if __name__ == '__main__':
    unittest.main()



import pandas as pd
import os

class Course:
    def __init__(self, course_name, file_path, course_record=None):
        """
        Initialize a Course object.

        Parameters
        ----------
        course_name : str
            The name of the golf course.
        file_path : str
            The file path to the course score card CSV file.
        course_record : int or None, optional
            The course record, if available. Default is None.
        """
        self.course_name = course_name
        try:
            score_card = pd.read_csv(file_path)
        except:
            raise ValueError("Error with file or file path, please try again.")
        self.score_card = score_card
        self.course_record = course_record
        try:
            par = sum(self.score_card['par'])
        except:
            raise ValueError("Could not read course par values, make sure they are integers")
        self.par = int(par)
        
    def add_course(course_name, file_path, course_record=None):
        """
        Create a new golf course object.

        Parameters
        ----------
        course_name : str
            The name of the golf course.
        file_path : str
            The file path to the course score card CSV file.
        course_record : int or None, optional
            The course record, if available. Default is None.

        Returns
        -------
        Course or str
            A Course object if successful, otherwise an error message as a string.
        """
        try:
            pd.read_csv(file_path)
        except:
            return "ERROR"
        return Course(course_name, file_path, course_record)
    
    def __str__(self):
        """
        Return a string representation of the Course object.
        """
        pd.set_option('display.max_rows', None)
        pd.set_option('display.max_columns', None)
        return f"\nCourse: {self.course_name} \nCourse Record: {self.course_record}\nPar: {self.par} \nScore Card:\n{self.score_card.T}\n"

script_dir = os.path.dirname(os.path.abspath(__file__))
script_dir = os.path.join(script_dir, 'course_files')

sunset_ranch_course_record = 65
csv_file_path = os.path.join(script_dir, "sunset_ranch.csv")
sunset_ranch_course = Course("Sunset Ranch",csv_file_path, sunset_ranch_course_record)

shadow_ridge_course_record = 64
csv_file_path = os.path.join(script_dir, "shadow_ridge.csv")
shadow_ridge_course = Course("Shadow Ridge", csv_file_path, shadow_ridge_course_record)

okanagan_golf_club_bear_course_record = 68
csv_file_path = os.path.join(script_dir, "okanagan_golf_club_bear.csv")
okanagan_golf_club_bear_course = Course("Okanagan Golf Club Bear",csv_file_path, okanagan_golf_club_bear_course_record)

pd.reset_option('display.max_rows')
pd.reset_option('display.max_columns')


import unittest
from golf_companion_533_app.__player_class import Player
# from __player_class import Player # delete this

class TestPlayerProfessional(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        print('Setup for TestPlayerProfessional')

    @classmethod
    def tearDownClass(cls):
        print('Teardown for TestPlayerProfessional')

    def setUp(self):
        self.player = Player(name="John Doe", skill=1)  # Professional skill level

    def tearDown(self):
        self.player = None

    def test_name(self):
        self.assertEqual(self.player.name, "John Doe")

    def test_skill(self):
        self.assertEqual(self.player.skill, 1)

    def test_yardages(self):
        self.assertEqual(self.player.yardages, Player.professional_yardages)

    def test_handicap(self):
        self.assertIsNone(self.player.handicap)


class TestPlayerAmateur(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        print('Setup for TestPlayerAmateur')

    @classmethod
    def tearDownClass(cls):
        print('Teardown for TestPlayerAmateur')

    def setUp(self):
        self.player = Player(name="Jane Doe", skill=3)  # Amateur skill level

    def tearDown(self):
        self.player = None

    def test_name(self):
        self.assertEqual(self.player.name, "Jane Doe")

    def test_skill(self):
        self.assertEqual(self.player.skill, 3)

    def test_yardages(self):
        amateur_yardages = {club: int(yardage * 0.6) for club, yardage in Player.professional_yardages.items()}
        self.assertEqual(self.player.yardages, amateur_yardages)

    def test_handicap(self):
        self.assertIsNone(self.player.handicap)


if __name__ == '__main__':
    unittest.main()



import unittest
from golf_companion_533_app.pick_club.start_picking import start_picking
from golf_companion_533_app.__player_class import Player 

class TestStartPickingLowYardage(unittest.TestCase):
    def setUp(self):
        self.player = Player(name="Test Player", skill=1)  # Assuming skill level 1 has all clubs

    def test_putter_selected(self):
        self.assertEqual(start_picking(15, self.player), "Putter")

    def test_no_club_for_negative_yards(self):
        with self.assertRaises(ValueError):
            start_picking(-10, self.player)

    def test_no_club_for_zero_yards(self):
        self.assertEqual(start_picking(0, self.player), "Putter")

    def test_invalid_input_type(self):
        with self.assertRaises(ValueError):
            start_picking("fifty", self.player)


class TestStartPickingHighYardage(unittest.TestCase):
    def setUp(self):
        self.player = Player(name="Test Player", skill=1)  # Assuming skill level 1 has all clubs

    def test_driver_selected(self):
        self.assertEqual(start_picking(305, self.player), "Driver")

    def test_no_suitable_club_found(self):
        self.player.yardages = {club: 20 for club in self.player.yardages}  # Set all yardages to 20
        self.assertEqual(start_picking(350, self.player), "Driver")

    def test_correct_club_for_mid_range(self):
        # Assuming '5 Iron' is the club for 195 yards
        self.assertEqual(start_picking(195, self.player), "5 Iron")

    def test_type_error_for_non_number(self):
        with self.assertRaises(ValueError):
            start_picking(None, self.player)

if __name__ == '__main__':
    unittest.main()



import unittest
import golf_companion_533_app
from golf_companion_533_app import player_test
from golf_companion_533_app import test_course
from golf_companion_533_app import track_score
from golf_companion_533_app.pick_club import pick_test


def my_suite():
    suite = unittest.TestSuite()
    result = unittest.TestResult()
    suite.addTest(player_test.TestPlayerProfessional('test_name'))
    suite.addTest(player_test.TestPlayerProfessional('test_skill'))
    suite.addTest(player_test.TestPlayerProfessional('test_yardages'))
    suite.addTest(player_test.TestPlayerProfessional('test_handicap'))
    
    suite.addTest(player_test.TestPlayerAmateur('test_name'))
    suite.addTest(player_test.TestPlayerAmateur('test_skill'))
    suite.addTest(player_test.TestPlayerAmateur('test_yardages'))
    suite.addTest(player_test.TestPlayerAmateur('test_handicap'))  
    
    suite.addTest(test_course.TestCourseOkanagan('test_course_name'))
    suite.addTest(test_course.TestCourseOkanagan('test_course_record'))
    suite.addTest(test_course.TestCourseOkanagan('test_course_par'))
    suite.addTest(test_course.TestCourseOkanagan('test_course_score_card_type'))
    
    suite.addTest(test_course.TestCourseShadowRidge('test_course_name'))
    suite.addTest(test_course.TestCourseShadowRidge('test_course_record'))
    suite.addTest(test_course.TestCourseShadowRidge('test_course_par'))
    suite.addTest(test_course.TestCourseShadowRidge('test_course_score_card_type'))   
    
    suite.addTest(pick_test.TestStartPickingLowYardage('test_putter_selected'))
    suite.addTest(pick_test.TestStartPickingLowYardage('test_no_club_for_negative_yards'))
    suite.addTest(pick_test.TestStartPickingLowYardage('test_no_club_for_zero_yards'))
    suite.addTest(pick_test.TestStartPickingLowYardage('test_invalid_input_type'))
    
    suite.addTest(pick_test.TestStartPickingHighYardage('test_driver_selected'))
    suite.addTest(pick_test.TestStartPickingHighYardage('test_no_suitable_club_found'))
    suite.addTest(pick_test.TestStartPickingHighYardage('test_correct_club_for_mid_range'))
    suite.addTest(pick_test.TestStartPickingHighYardage('test_type_error_for_non_number'))
    
    runner = unittest.TextTestRunner()
    print(runner.run(suite))
my_suite()


from golf_companion_533_app import __player_class

def start_picking(target_yards, player):
    if not isinstance(target_yards, (int, float)) or target_yards < 0:
        raise ValueError("Target yards must be a positive number.")

    # Check for special cases first
    if target_yards < 20:
        return "Putter"
    elif target_yards > 300:
        return "Driver"

    best_club = None
    min_diff = float('inf')

    for club, yardage in player.yardages.items():
        # Skip the putter and driver since we have already handled them
        if club in ["Putter", "Driver"]:
            continue

        diff = target_yards - yardage
        if 0 <= diff < min_diff:
            best_club = club
            min_diff = diff

    if best_club is None:
        return "No suitable club found."

    return best_club



#import golf_companion.start_game
#import golf_companion.pick_club
#import golf_companion.track_score
# from golf_companion.track_score.start_tracking import start_tracking
# from golf_companion.pick_club.start_picking import start_picking



import sys
import pandas as pd

from golf_companion_533_app.pick_club import start_picking
from golf_companion_533_app.track_score import start_tracking
from golf_companion_533_app import __player_class


def start_game(players=[], course=None, num_holes=None):
    """
    Start a game of golf. If player or course or num_holes objects/values are passed, then the 
    user prompts for these information is skipped. If a list of player objects is passed then 
    the option to add additional/new players still remains. 

    Parameters
    ----------
    players : list of __player_class.Player, optional
        List of players. Default is an empty list.
    course : __course_class.Course or None, optional
        Golf course object. Default is None.
    num_holes : int or None, optional
        Number of holes to track. Default is None.

    Returns
    -------
    list
        List containing players, the course, and final scores.
    """
    try:
        # Prompts for course, players, and number of holes to play
        if course is None:
            course = start_tracking.__choose_course()
        if not players:
            players = start_tracking.__add_game_players()
        else:  # if players are passed then prompt to check if new players would like to be added
            for k in players:
                k.score = {}
            ask_to_add_players = None
            while ask_to_add_players is None:
                try:
                    print(f"\nPlayers in game:", end="")
                    for player in players:
                        print(f" {player.name}", end="")
                    ask_to_add_players = input("\nDo you want to add more players: \n1: Yes \n2: No\n")
                    if start_tracking.__check_value_is_number(ask_to_add_players):
                        if int(ask_to_add_players) == 1:
                            players = start_tracking.__add_game_players(players)
                        elif int(ask_to_add_players) == 2:
                            ask_to_add_players = False
                        else:
                            ask_to_add_players = None
                    else:
                        raise ValueError("Invalid input. Please enter 1 or 2.")
                except ValueError as ve:
                    print(f"Error: {ve}")
                    ask_to_add_players = None

        if num_holes is None:
            num_holes = start_tracking.__check_num_holes()

        # Starting the game tracking - will start by repeating club help prompt until no more help is required.
        # Usually players would play the full hole (i.e., may require club help) and then enter the final score once
        # the hole is complete.
        for i in range(num_holes):
            border = "-" * len(f"Hole {i + 1} | Par {course.score_card['par'][i]} | {course.score_card['yards'][i]} yards")
            print("\n" + border)
            print(f"Hole {i + 1} | Par {course.score_card['par'][i]} | {course.score_card['yards'][i]} yards")
            print(border + "\n")

            pick_club_option = None
            while pick_club_option is None:
                try:
                    pick_club_option = input("Do you need help picking a club: \n1: Yes \n2: No \n3: Exit \n")
                    print("")
                    check_pick_club_option = start_tracking.__check_value_is_number(pick_club_option)
                    if check_pick_club_option:
                        if int(pick_club_option) == 3:
                            start_tracking.__exit()
                        elif int(pick_club_option) == 1:
                            pick_club_option = None
                            which_player = None
                            while which_player is None:
                                try:
                                    print("Which player needs help picking a club?")
                                    for j in range(len(players)):
                                        print(f"{j+1}: {players[j].name}")
                                    which_player = input()
                                    check_which_player = start_tracking.__check_value_is_number(which_player)
                                    if check_which_player:
                                        if 0 < int(which_player) <= len(players):
                                            yardage = None
                                            while yardage is None:
                                                try:
                                                    yardage = input("\nEnter yardage to the flag: ")
                                                    check_yardage = start_tracking.__check_value_is_number(yardage)
                                                    if check_yardage:
                                                        if 0 < int(yardage) < 750:
                                                            club = start_picking.start_picking(int(yardage),
                                                                                                players[int(which_player) - 1])
                                                            print(f"{players[int(which_player) - 1].name} hit your {club}\n")
                                                        else:
                                                            print("Yardage should be greater than 0 and less than 750 yards.")
                                                            yardage = None
                                                    else:
                                                        which_player = None
                                                except ValueError as ve:
                                                    print(f"Error: {ve}")
                                                    yardage = None
                                        else:
                                            which_player = None
                                except ValueError as ve:
                                    print(f"Error: {ve}")
                                    which_player = None
                        elif int(pick_club_option) == 2:
                            pick_club_option = int(pick_club_option)
                        else:
                            raise ValueError("Invalid input. Please enter 1, 2, or 3.")
                    else:
                        raise ValueError("Invalid input. Please enter a number.")
                except ValueError as ve:
                    print(f"Error: {ve}")
                    pick_club_option = None

            # Tracking the final score of all players for a particular hole.
            players = start_tracking.__track_hole(players, course, i, False)

        print_summary = start_tracking.__print_summary(players, course)
        final_scores = print_summary
        min_score_index = final_scores.index(min(final_scores))
        tie = False
        for i in range(len(final_scores)):
            if final_scores[min_score_index] == final_scores[i] and i != min_score_index:
                tie = True
        if tie:
            print(f"\nThere was a tie! The following players tied for a final score of {final_scores[min_score_index]}!")
            indices = [index for index, item in enumerate(final_scores) if item == final_scores[min_score_index]]
            for i in indices:
                print(f"{players[i].name}")
        else:
            print(
                f"\nGame winner is {players[min_score_index].name} who got a final score of {final_scores[min_score_index]}!")
        print("")
        print("End of game...")
        return [players, course, final_scores]

    except Exception as e:
        print(f"Error in start_game function: {e}")
        # Optionally, you might want to log the exception for further investigation.
        # logging.exception(f"Error in start_game function: {e}")

# Example usage:
# b = start_game()



import sys
import pandas as pd

# import __course_class
from golf_companion_533_app import __course_class
from golf_companion_533_app import __player_class

def __exit():
    """Raise SystemExit to exit the program."""
    raise SystemExit("\nExiting the program")
    sys.exit()

def __check_value_is_number(value):
    """
    Check if the input from user is a valid number.

    Parameters
    ----------
    value : str
        The input value to check.

    Returns
    -------
    bool
        True if the value is a number, False otherwise.
    """   
    try:
        int(value)
    except:
        return False
    return True

def __add_new_course():
    """
    Add a new golf course. Prompts the user for file location of the csv storing the 
    score card of the new course.

    Returns
    -------
    __course_class.Course
        A new course object.
    """
    file_path = None
    while file_path == None:
        file_path = input("\nEnter file path to course score card: \n\n*Note: File must be csv and have the column headings 'hole' | 'yards' | 'par' - in that order.\n\nPress 5 if you want to exit. \n")
        
        check_file_path = __check_value_is_number(file_path)
        if check_file_path == True and int(file_path) == 5:
            __exit()
        
        try:
            pd.read_csv(file_path)
        except:
            file_path = None
            print("\nCould not find or read file, try again. \n")
        if file_path != None:
            course_name = input("\nEnter a name for the course: ")
            course_record = input("\nEnter the course record, if you know it, or enter 0: \n*Note: Any value other than 0 or a valid integer will not be used.\n")
    
    if course_name == "" or course_name == " ":
        course_name = None
    check_course_record = __check_value_is_number(course_record)
    
    if not course_record.isnumeric() or int(course_record) == 0:
        course_record = None   
    new_course = __course_class.Course.add_course(course_name, file_path, course_record)
    
    return new_course

def __choose_course():
    """
    Choose a golf course - can also add a new course.

    Returns
    -------
    __course_class.Course or None
        The chosen course object or None if the user exits.
    """
    course = None
    while course == None:
        course = input("\nWhat course would you like to play? \nPress: \n1: Sunset Ranch \n2: Shadow Ridge \n3: Okanagan Golf Club Bear \n4: Load Your own course\n5: Exit \n")
        check_course = __check_value_is_number(course)
        if check_course == True:
            course = int(course)
        else:
            course = None
        if course not in [1,2,3,4,5]:
                print("\nPlease choose one of the listed options")
                course = None
    if course == 5:
        __exit()
    elif course == 1:
        return __course_class.sunset_ranch_course
    elif course == 2:
        return __course_class.shadow_ridge_course
    elif course == 3:
        return __course_class.okanagan_golf_club_bear_course
    elif course == 4:
        return __add_new_course()
    return None

def __add_player():
    """
    Add a new player, prompts for player name, skill, and handicap.

    Returns
    -------
    __player_class.Player
        A new player object.
    """
    try:
        player_name = None
        while player_name is None or player_name == "" or player_name == " " or not player_name.replace(" ", "").isalpha():
            try:
                player_name = input("\nEnter player name: \n*Note: Cannot be empty and only alphabets allowed. \n")
            except Exception as e:
                print(f"Error getting player name: {e}")

        player_skill = None
        while player_skill is None or player_skill == "" or player_skill == " ":
            try:
                player_skill = input("\nChoose player skill. \nEnter: \n1: Professional \n2: Intermediate \n3: Amateur \n0: Exit \n")
                if __check_value_is_number(player_skill):
                    if int(player_skill) not in [0, 1, 2, 3]:
                        player_skill = None
                    elif int(player_skill) == 0:
                        __exit()
                else:
                    player_skill = None
            except Exception as e:
                print(f"Error getting player skill: {e}")

        player_handicap = None
        while player_handicap is None:
            try:
                player_handicap = input("\nEnter player handicap (-10 < H < 30) or press Enter to skip \n*Note: Handicap must be a number \n")
                if player_handicap != "" and player_handicap != " " and not player_handicap.isnumeric():
                    player_handicap = None
                if __check_value_is_number(player_handicap):
                    if int(player_handicap) < -10 or int(player_handicap) > 30:
                        player_handicap = None
            except Exception as e:
                print(f"Error getting player handicap: {e}")

        check_handicap = __check_value_is_number(player_handicap)
        if not check_handicap:
            player_handicap = None
        else:
            player_handicap = int(player_handicap)

        player = __player_class.Player(player_name, int(player_skill), player_handicap)
        return player

    except Exception as e:
        print(f"Error adding player: {e}")
        return None


def __track_hole(players, course, hole_num, print_header = True):
    """
    Track score for all players for n number of holes (hole_num).

    Parameters
    ----------
    players : list of __player_class.Player
        List of players.
    course : __course_class.Course
        Golf course object.
    hole_num : int
        Current hole number.
    print_header : bool, optional
        Whether to print the hole header. Default is True.

    Returns
    -------
    list of __player_class.Player
        Updated list of players.
    """
    if print_header == True:
        check = input("\nEnter 0 to exit, anything else will continue: ")
        if __check_value_is_number(check) == True:
            if int(check) == 0:
                __exit()
        border = "-" * len(f"\nHole {hole_num+1} | Par {course.score_card['par'][hole_num]} | {course.score_card['yards'][hole_num]} yards\n")
        print("\n" + border)
        print(f"Hole {hole_num+1} | Par {course.score_card['par'][hole_num]} | {course.score_card['yards'][hole_num]} yards")
        print(border + "\n")
    for player in players:
        if player.score == None:
            player.score = {}
        player_score = None
        while player_score == None:
            player_score = input(f"Enter the final score for {player.name}: ")
            if __check_value_is_number(player_score) == False:
                player_score = None
            else:
                player.score.update({hole_num+1: int(player_score)})
    return players

def __print_summary(players, course):
    """
    Print a summary of scores for the round. Can either print a detailed summary of scores by hole
    or print a condensed version.

    Parameters
    ----------
    players : list of __player_class.Player
        List of players.
    course : __course_class.Course
        Golf course object.

    Returns
    -------
    list
        List of final scores.
    """
    print_summary = None
    final_score = []
    total_score = []
    adjusted_score = []
    for i in range(len(players)):
        adjusted_score.append(None)
    while print_summary == None:
        print_summary = input("\nDo you wish to see a full breakdown of scores?\n1: Yes \n2: No \n3: Exit \n")
        if __check_value_is_number(print_summary) == True:
            if int(print_summary) == 3:
                __exit()
            elif int(print_summary) not in [1,2]:
                print_summary = None
    for i in range(len(players)):
        total_score.append(sum(players[i].score.values()))
        if players[i].handicap != None:
            adjusted_score[i] = sum(players[i].score.values()) - int(players[i].handicap)
    if int(print_summary) == 1:
        border = "-" * len(f"{course.course_name} | Par {course.par} | Course Record: {course.course_record}")
        print("\n" + border)
        print(f"{course.course_name} | Par {course.par} | Course Record: {course.course_record}")
        print(border)
        for i in range(len(players)):
            print(players[i])
            print(f"\nTotal Score: {total_score[i]}")
            if players[i].handicap != None:
                print(f"Adjusted Score: {adjusted_score[i]}")
    else:
        border = "-" * len(f"{course.course_name} | Par {course.par} | Course Record: {course.course_record}")
        print("\n" + border)
        print(f"{course.course_name} | Par {course.par} | Course Record: {course.course_record}")
        print(border)
        for i in range(len(players)):
            print(f"\nPlayer: {players[i].name} \nTotal Score: {total_score[i]} | Adjusted Score: {adjusted_score[i]}")
    for i in range(len(players)):
        if adjusted_score[i] == None:
            final_score.append(total_score[i])
        else:
            final_score.append(adjusted_score[i])
    return final_score

def __check_num_holes():
    """
    Check the number of holes to track.

    Returns
    -------
    int
        Number of holes.
    """
    num_holes = None
    while num_holes == None:
        num_holes = input("\nHow many holes are you going to track. Enter '0' to exit: ")
        check_num_holes = __check_value_is_number(num_holes)
        if check_num_holes == True:
            if int(num_holes) == 0:
                __exit()
            else:
                if int(num_holes) > 72 or int(num_holes) < 0:
                    print("Must track atleast 1 hole and cannot track more than 4 full games (72 holes) of golf at 1 time.")
                    num_holes = None
                else:
                    num_holes = int(num_holes)
        else:
            num_holes = None
    return num_holes

def __add_game_players(players=[]):
    """
    Add players to the game.

    Parameters
    ----------
    players : list of __player_class.Player, optional
        List of players. Default is an empty list.

    Returns
    -------
    list of __player_class.Player
        Updated list of players.
    """
    if players == []:
        players = []
    num_players = "a"
    while num_players.isnumeric() == False:
        num_players = input("\nEnter the number of players to add (max 6) or enter '0' to exit: ")
        if __check_value_is_number(num_players) == True:
            if int(num_players) == 0:
                __exit()
            elif int(num_players) > 6 or int(num_players) < 0:
                num_players = "a"
    players_already_in_game = len(players)
    for i in range(int(num_players)):
        border = "-" * len(f"Player {i+1}")
        print("\n" + border)
        print(f"Player {i + 1 + players_already_in_game}")
        print(border)
        players.append(__add_player())
    return players

def start_tracking(players=None, course=None, num_holes=None):
    """
    Track scores for a golf game.

    Parameters
    ----------
    players : list of __player_class.Player, optional
        List of players. Default is an empty list.
    course : __course_class.Course or None, optional
        Golf course object. Default is None.
    num_holes : int or None, optional
        Number of holes to track. Default is None.

    Returns
    -------
    list
        List containing players and the course.
    """
    try:
        if course is None:
            course = __choose_course()
        if course is None:
            print("No Course Chosen!")
        if players is None:
            players = __add_game_players()
        else:
            ask_to_add_players = None
            while ask_to_add_players is None:
                try:
                    print(f"\nPlayers in the game:", end="")
                    for player in players:
                        print(f" {player.name}", end="")
                    ask_to_add_players = input("\nDo you want to add more players: \n1: Yes \n2: No\n")
                    if __check_value_is_number(ask_to_add_players):
                        if int(ask_to_add_players) == 1:
                            players = __add_game_players(players)
                        elif int(ask_to_add_players) == 2:
                            ask_to_add_players = False
                        else:
                            ask_to_add_players = None
                    else:
                        ask_to_add_players = None
                except Exception as e:
                    print(f"Error processing player input: {e}")

        if num_holes is None:
            num_holes = __check_num_holes()

        for i in range(num_holes):
            players = __track_hole(players, course, i)

        print_summary = __print_summary(players, course)

        print("\nEnd of tracking...")

        return [players, course]

    except Exception as e:
        print(f"Error in start_tracking: {e}")
        return None


